/*
** pork_aim.h - pork's interface with libfaim
** Copyright (C) 2002-2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#ifndef __PORK_AIM_H
#define __PORK_AIM_H

#define AIM_DEFAULT_CHAT_EXCHANGE	4

#define DEFAULT_AIM_PROFILE "Cogito ergo <a href=\"http://dev.ojnk.net\">HAM</a>."

#define FAIM_CB(x) int x(aim_session_t *session __notused, aim_frame_t *fr __notused, ...)

struct buddy;
struct bgroup;

struct chatroom {
	char *title;
	char *name;
	char *description;
	u_int16_t exchange;
	u_int16_t instance;
	aim_conn_t *conn;
	int max_msg_len;
	int max_visible_len;
	size_t num_users;
	time_t created;
	struct imwindow *win;
	dlist_t *user_list;
};

struct chatroom_info {
	char *name;
	u_int16_t exchange;
};

int aim_setup(struct aim_priv *priv);

int aim_login(struct aim_priv *priv);
int aim_send_msg(struct aim_priv *priv, char *dest, char *msg);
int aim_add_block(struct aim_priv *priv, char *target);
int aim_remove_block(struct aim_priv *priv, char *target);
int aim_add_permit(struct aim_priv *priv, char *target);
int aim_remove_permit(struct aim_priv *priv, char *target);
int aim_buddy_add(struct aim_priv *priv, struct buddy *buddy);
int aim_buddy_remove(struct aim_priv *priv, struct buddy *buddy);
int aim_buddy_move(struct aim_priv *priv, struct buddy *buddy, char *old_group);
int aim_group_add(struct aim_priv *priv, struct bgroup *group);
int aim_group_remove(struct aim_priv *priv, struct bgroup *group);
int aim_warn(struct aim_priv *priv, char *target);
int aim_warn_anon(struct aim_priv *priv, char *target);
int aim_warn_anon(struct aim_priv *priv, char *target);
int aim_set_away(struct aim_priv *priv, char *away_msg);
int aim_keepalive(struct aim_priv *priv);
int aim_set_idle(struct aim_priv *priv, int idle_secs);
int aim_set_profile(struct aim_priv *priv, char *profile);
int aim_get_away_msg(struct aim_priv *priv, char *buddy);
int aim_whois(struct aim_priv *priv, char *buddy);
void aim_kill_all_conn(struct aim_priv *priv);
int aim_permdeny_mode(struct aim_priv *priv, int mode);
int aim_report_idle(struct aim_priv *priv, int mode);

void aim_chat_print_users(struct imwindow *win, struct chatroom *chat);
dlist_t *aim_find_chat(struct aim_priv *, const char *name, int exchange);
dlist_t *aim_find_chat_name(struct aim_priv *priv, char *name);
void aim_update_buddy(struct buddy *buddy, aim_userinfo_t *userinfo);
int aim_join_chatroom(struct aim_priv *priv, char *name, int exchange);
int aim_leave_chatroom(struct aim_priv *priv, char *name, int exchange);
int aim_chat_send(struct aim_priv *priv, char *msg, struct chatroom *chat);
int aim_chat_ignore(char *user, struct chatroom *chat);
int aim_chat_unignore(char *user, struct chatroom *chat);

int aim_chat_send_name(	struct aim_priv *priv,
						char *msg,
						char *name,
						int exchange);

int aim_send_buddy_list(	aim_session_t *session,
							aim_conn_t *conn,
							struct aim_priv *priv);

int aim_send_permit_list(	aim_session_t *session,
							aim_conn_t *conn,
							struct aim_priv *priv);

int aim_send_block_list(	aim_session_t *session,
							aim_conn_t *conn,
							struct aim_priv *priv);

int aim_chat_send_invite(	struct aim_priv *priv,
							char *dest,
							char *msg,
							struct chatroom *chat);

#endif
